#! /usr/bin/python

## corpus.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    corpus
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

corpus
======

"""


from pprint import pprint # debug

from trefnydd.utils import file_IO, location
from utterance import *

class Corpus(list, location.Located):
    def __init__(self, debug=False):
        location.Located.__init__(self, debug)
        self.dataDir = 'Data'
        self.name =  ''
        self.indices = {} # labelType: [instances]

    def save(self):
        import os
        fullPath = file_IO.getDir(os.path.abspath(os.path.join(self.homeDir,
                                                               self.name,
                                                               self.dataDir)))
        currDir = os.getcwd()
        os.chdir(fullPath)
        if self.debug is True:
            print """corpus.save:
    self.homeDir = %s
    self.name = %s
    self.dataDir = %s\n
    fullPath = \n\t%s\n""" \
                % (self.homeDir, self.name, self.dataDir, fullPath)
        for item in self:
            tmp = os.path.join(fullPath, item.name)
            item.homeDir = tmp
            if self.debug is True:
                print 'tmp = %s\ncorpus.save: item.homeDir = \n\t%s\n' \
                      % (tmp, item.homeDir)
            item.save()
        # anything else to save?
        os.chdir(currDir)

    def makeIndex(self, labelType):
        self.indices[labelType] = {}
        for item in self:
            # todo: make a generic/morew abstract version
            # of speechCorpus.makeIndex(labelType)
            # see find
            pass

    def find(self, labelType, label):
        """
        returns list of items (utts, sentences) which contain label in labelType tier
        """
        pass


class TextCorpus(Corpus): # list of Sentences
    # cribbed from wispr::Optese
    def __init__(self, debug=False):
        Corpus.__init__(self, slist)
        self.debug = debug
        self.freqs = {}
        self.freq_order = []

    def __add__(self, other):
        self.extend(other[:])
        self.reset_freqs()
        return self

    def __sub__(self, other):
        for s in other[:]:
            if s in self:
                self.remove(s)
        self.reset_freqs()
        return self

    def add_sentence(self, s):
        self.append(s)
        for dp in s:
            self.freqs[dp] = self.freqs.setdefault(dp, 0) + s[dp]
        self.set_freq_order

    def remove_sentence(self, s):
        if s in self:
            self.remove(s)
        for dp in s:
            self.freqs[dp] = self.freqs.setdefault(dp, 0) - s[dp]
        self.set_freq_order

    def get_sentence_by_ID(self, sid):
        # improve this!
        s = [s for s in self if s.src_id == sid][0]
        return s
            
    def set_freqs(self):
        self.freqs = {}
        for s in self:
            for dp in s:
                self.freqs[dp] = self.freqs.setdefault(dp, 0) + s[dp]

    def set_freq_order(self):
        self.freq_order = list(set(self.freqs.values()))
        self.freq_order.sort()
    
    def reset_freqs(self):
        self.set_freqs()
        self.set_freq_order()

        
    
class SpeechCorpus(Corpus):
    def __init__(self, debug=False):
        Corpus.__init__(self, debug)
        self.debug = debug
        self.pd = None # Pronunciation Dictionary
        self.pdFn = ''

        self.freqs = {}
        self.freq_order = []

    def __add__(self, other):
        self.extend(other[:])
        self.reset_freqs()
        return self

    def __sub__(self, other):
        for s in other[:]:
            if s in self:
                self.remove(s)
        self.reset_freqs()
        return self

    def importData(self, dataDir=''):
        if not dataDir: dataDir = self.homeDir
        fstems = file_IO.get_fstems(dataDir)
        for fstem in fstems:
            utt = Utterance(os.path.join(dataDir, fstem))
            utt.homeDir = dataDir
            self.add_utterance(utt)

    def load(self, corpusDir):
        if self.debug is True:
            print 'SpeechCorpus.load'
        if corpusDir[-1] == '/': corpusDIr = corpusDir[:-1]
        home, name = os.path.split(corpusDir)
        self.homeDir = home
        self.name = name
        # todo: DANGER: magic text 'Data'
        # todo: any non-data dirs?
        dataDir = os.path.join(corpusDir, 'Data')
        for dirn in os.listdir(dataDir):
            utt = Utterance()
            utt.debug = self.debug
            utt.name = dirn
            utt.homeDir = os.path.abspath(os.path.join(dataDir, dirn))
            utt.load()
            self.add_utterance(utt)

    def add_utterance(self, utt):
        self.append(utt)

    def remove_utterance(self, utt):
        if utt in self:
            self.remove(utt)
        for label in utt:
            self.freqs[label] = self.freqs.setdefault(label, 0) - utt[label]
        self.set_freq_order

    def get_utterance_by_ID(self, sid):
        # improve this!
        utt = [s for s in self if s.src_id == sid][0]
        return utt
            
    def set_freqs(self):
        self.freqs = {}
        for utt in self:
            for label in utt:
                self.freqs[label] = self.freqs.setdefault(label, 0) \
                                    + utt[label]

    def set_freq_order(self):
        self.freq_order = list(set(self.freqs.values()))
        self.freq_order.sort()
    
    def reset_freqs(self):
        self.set_freqs()
        self.set_freq_order()

    def makeIndex(self, labelType):
        self.indices[labelType] = {}
        for utt in self:
            tier = utt.getTierByName(labelType)
            for seg in tier:
                self.indices[labelType].setdefault(seg.label, 0)

    def get_labelTypes(self):
        labelTypes = {}
        for utt in self:
            for tier in utt.tiers:
                labelTypes.setdefault(tier.name, 0)
        return labelTypes.keys()
                


def main():
    dataDir = 'raw_data'
    sc = SpeechCorpus()
    sc.importData(dataDir)
    lt = sc.get_labelTypes()[0]
    sc.makeIndex(lt)
    pprint(sc.indices[lt].keys())

if __name__ == '__main__':
    main()
