#! /usr/bin/python

## labelFileInterface.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    labelFileInterface
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

labelFileInterface
==================

"""

#from trefnydd.utils import location

class LabelFileInterface(object) #location.Located):
    def __init__(self, debug=False):
        location.Located.__init__(self, debug)

    def read(self, fn):
        """\
        Reads label file into an SegmentationTier object.
        Returns: SegmentationTier
        """
        pass

    def write(self, st):
        """\
        Serialises SegmentationTier object.
        Returns: string
        """
        pass

    def getHeadBody(self, fn, sep):
        data = list(open(fn))
        # lines with misc whitespace --> '\n'
        # TODO: or with misc whitespace at end of line
        for i in range(len(data)):
            if data[i].strip == '':
                data[i] = '\n'
        hidx = data.index(sep) + 1
        head, body = data[0:hidx], data[hidx:]
        return head, body

    

class ESPSFileInterface(LabelFileInterface):
    def __init__(self):
        LabelFileInterface.__init__(self)

    def read(self, fn):
        pass

    def write(self, st):
        pass

    def read(self, fn):
        """\
        Reads label file into an SegmentationTier object.
        Returns: SegmentationTier
        # utterance:
        Returns ["head", [time, something_else, label]]
        """
        head, body = self.getHeadBody(fn, '#\n')
        
        self.parseHead(head, 'esps')
        tier = SegmentationTier()
        tier.setName('Phone') # TODO: magic text!
        if body:
            for line in body:
                fields = line.split()
                # fields[0] = time
                # fields[1] = something_else  - TODO: Q: what is it?
                # fields[2] = label
                if len(fields) != 3:
                    if self.debug is True:
                        print 'DEBUG: N FIELDS: %s line: %s' % (fn, line)
                        print tier
                    #else:
                    pass # what then!?
                else:
                    seg = Segment()
                    if tier:
                        seg.min = tier[-1].max
                    else: seg.min = 0
                    seg.max = eval(fields[0])
                    seg.label = fields[-1]
                    tier.append(seg)
            self.updateTiers(tier)

    def write_ESPS(self, format, tier='Phone'):
        ### TODO: magic text! only one tier!
        out = '%s\n' % (self.writeHead(format))
        # what about writing multiple tiers?
        for segment in self.getTierByName(tier):
            out = '%s\t%s\t125 %s\n' \
                  % (out, segment.max, segment.label)
        # todo: tier separator in esps format!?
        return out

    

def main():
    pass

if __name__ == '__main__':
    main()
 
