#! /usr/bin/python

## file_IO.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    file_IO
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

file_IO
=======

"""


import os, sys, types

#####  read  #####

def read_stdin_2_list():
    in_list = []
    raw_in = sys.stdin.read()
    in_list.extend(raw_in.split('\n'))
    return in_list

def read_file_2_list(fn, toList=True):
    in_list = []
    for line in [i for i in open(fn)
                 if i[0] not in '\n#']: # ignore blank lines and comments
        line = line.strip() # was: if line[-1] == '\n': line = line[:-1]
        if '\t' in line and toList is True: # line -> list
            line = line.split('\t')
        in_list.append(line)
    return in_list

def read_file_2_dict(dict_fn, rm_count=0):
    in_dict = {}
    last_key = ''
    for line in open(dict_fn, 'r'):
        if line[0] == '\t':
            line = last_key + line
        key, val = line.split()[0], line.split()[1:]
        if rm_count:
            if key[-1] == ')':
                key = key[:key.index('(')]
        in_dict.setdefault(key, []).append(val)
    return in_dict

def get_fstems(dirn):
    fstems = {}
    for stem in [os.path.splitext(f)[0] for f in os.listdir(dirn)]:
        fstems.setdefault(stem, 0)
    fstems = fstems.keys()
    fstems.sort()
    return fstems

def getDir(dirn):
    if not os.path.exists(dirn):
        os.makedirs(dirn)
    return dirn


#####  write  #####

def write_list_2_file(list, fn, arg='w'):
    fp = open(fn, arg)
    for item in list:
        try:
            item + ''
        except:
            line = str(item[0])
            for e in item[1:]:
                line += '\t' + str(e)
            item = line
        if len(item):
            if item[-1] != '\n':
                item += '\n'
        fp.write(item)
    fp.close()

def append_list_2_file(list, fn):
    write_list_2_file(list, fn, 'a')

def write_list_2_stdout(list):
    for item in list:
        try:
            item + ''
        except:
            line = str(item[0])
            for e in item[1:]:
                line += '\t' + str(e)
            item = line
        print item

def write_dict_2_file(dict, fn, with_count=0):
    fp = open(fn, 'w')
    items = dict.items()
    items.sort()
    klen = [len(x) for x in dict.keys()]
    klen.sort()
    col1w = klen[-1] + 6
    for item in items:
        if with_count:
            count = 0
        for val in item[1]:
            head = item[0]
            if with_count and count:
                head += '(' + str(count+1) + ')'
            line = '%-*s%s\n' % (col1w, head, str(val))
            fp.write(line)
            if with_count:
                count += 1
    fp.close()


if __name__ == '__main__':
    pass
#
